/**
 * \addtogroup 24XX256
 * @{
 */

/*****************************************************************************/
/**
 * \file 24xx256.h
 * 24xx256 serial eeprom routines for 8051 and Keil C51.
 * \author Copyright (c) 2010, Murray R. Van Luyn. <vanluynm@iinet.net.au>
 * \version 0.0
 * \date 04-07-10
 */
/*****************************************************************************/

/*****************************************************************************
 *
 *     THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 *     OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *     WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *     ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 *     DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *     DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 *     GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *     INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *     WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *     NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *     SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************/


#ifndef _24XX256_H
#define _24XX256_H

#include "I2C.h"


/*****************************************************************************
 *
 *                               Defined Constants
 *
 *****************************************************************************/

/** 
 * 24xx256 serial eeprom device memory page size in bytes.
 * \hideinitializer
 */
#define _24XX256_WORDS_PER_PAGE   64u

/** 
 * 24xx256 serial eeprom device number of memory pages.
 * \hideinitializer
 */
#define _24XX256_TOTAL_PAGES      512u

/** 
 * 24xx256 serial eeprom device number of memory words.
 * \hideinitializer
 */
#define _24XX256_TOTAL_WORDS      32768u

/** 
 * 24xx256 serial eeprom device identification code.
 * \hideinitializer
 */
#define _24XX256_EEPROM_ID        0xA0

/** 
 * 24xx256 device address word, physical device 0 address.
 * \hideinitializer
 */
#define _24XX256_DEVICE_0         0x00

/** 
 * 24xx256 device address word, physical device 1 address.
 * \hideinitializer
 */
#define _24XX256_DEVICE_1         0x01

/** 
 * 24xx256 device address word, physical device 2 address.
 * \hideinitializer
 */
#define _24XX256_DEVICE_2         0x02

/** 
 * 24xx256 device address word, physical device 3 address.
 * \hideinitializer
 */
#define _24XX256_DEVICE_3         0x03

/** 
 * 24xx256 device address word, physical device 4 address.
 * \hideinitializer
 */
#define _24XX256_DEVICE_4         0x04

/** 
 * 24xx256 device address word, physical device 5 address.
 * \hideinitializer
 */
#define _24XX256_DEVICE_5         0x05

/** 
 * 24xx256 device address word, physical device 6 address.
 * \hideinitializer
 */
#define _24XX256_DEVICE_6         0x06

/** 
 * 24xx256 device address word, physical device 7 address.
 * \hideinitializer
 */
#define _24XX256_DEVICE_7         0x07

/** 
 * 24xx256 serial eeprom device read flag.
 * \hideinitializer
 */
#define _24XX256_READ             0x01

/** 
 * 24xx256 serial eeprom device write flag.
 * \hideinitializer
 */
#define _24XX256_WRITE            0x00


/*****************************************************************************
 *
 *                           Public Function Prototypes
 *
 *****************************************************************************/

/** 
 * Serial eeprom initialisation function.
 * Call once at program start to initialise serial eeprom module.
 * \param -
 * \return -
 */
extern void _24xx256_init(void);

/** 
 * Serial eeprom byte-level read function.
 * Returns the single byte content of a stored serial eeprom memory word,
 * from the memory word address selected from the 32768 available.
 * Device selection is from the 8 addressable 24xx256 devices available.
 * \param device An \b unsigned \b char selecting desired eeprom device.
 * \param address An \b unsigned \b int indicating which memory word to read
 * from.
 * \return The retrieved memory word byte as an \b unsigned \b char.
 */
extern unsigned char _24xx256_read_byte(unsigned char device,
                                        unsigned int address);

/** 
 * Serial eeprom next byte-level read function.
 * Returns the single byte content of a stored serial eeprom memory word,
 * from the device buffered next memory word address in the 32768 available.
 * Device selection is from the 8 addressable 24xx256 devices available.
 * \param device An \b unsigned \b char selecting desired eeprom device.
 * \return The retrieved memory word byte as an \b unsigned \b char.
 */
extern unsigned char _24xx256_read_next_byte(unsigned char device);

/** 
 * Serial eeprom page-level read function.
 * Returns the requested number of stored serial eeprom memory words, up to
 * a full memory page, from the memory page number selected from the 512 
 * available. Byte retrieval continues past the singular requested memory page
 * if any more than a full page (64-bytes) of memory words are requested. 
 * Byte retrieval wraps-around to the first memory element if continued
 * beyond the last. 
 * Device selection is from the 8 addressable 24xx256 devices available.
 * \param device An \b unsigned \b char selecting desired eeprom device.
 * \param page An \b unsigned \b char indicating the memory page to read from.
 * \param num_bytes An \b unsigned \b char indicating the number of memory
 * words to read.
 * \param buffer An \b unsigned \b char pointer to the data buffer to read 
 * page bytes into.
 * \return -
 */
extern void _24xx256_read_page(unsigned char device,
                               unsigned char page,
                               unsigned char num_bytes, 
                               unsigned char * buffer);

/** 
 * Serial eeprom byte-level write function.
 * Writes a single byte to a serial eeprom memory word, at the memory word 
 * address selected from the 32768 available. 
 * This function will block for up to approximately 5ms, while the scheduled
 * serial eeprom byte write operation completes. 
 * Device selection is from the 8 addressable 24xx256 devices available.
 * \param device An \b unsigned \b char selecting desired eeprom device.
 * \param address An \b unsigned \b int indicating which memory word address
 * to write to.
 * \param byte An \b unsigned \b char to write to the addressed memory word.
 * \return -
 */
extern void _24xx256_write_byte(unsigned char device,
                                unsigned int address,
                                unsigned char byte);

/** 
 * Serial eeprom page-level write function.
 * Writes the requested number of serial eeprom memory words, up to a
 * full memory page, to the memory page number selected from the 512 available.
 * Subsequent byte storage wraps-around the singular specified memory page
 * if more than a full page (64-bytes) of memory words are delivered.
 * This function will block for up to approximately 5ms, while the scheduled
 * serial eeprom page write operation completes. 
 * Device selection is from the 8 addressable 24xx256 devices available.
 * \param device An \b unsigned \b char selecting desired eeprom device.
 * \param page An \b unsigned \b char indicating the memory page to address.
 * \param num_bytes An \b unsigned \b char indicating number of memory words
 * to write.
 * \param buffer An \b unsigned \b char pointer to the data bytes to write.
 * \return -
 */
extern void _24xx256_write_page(unsigned char device,
                                unsigned char page,
                                unsigned char num_bytes,
                                unsigned char * buffer);


#endif

/** @} */
